@@ -6,7 +6,8 @@ use warnings;
 
 use Scalar::Util 'blessed';
 
-our $VERSION = '0.01';
+our $VERSION   = '0.02';
+our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Class::MOP::Package';
 
@@ -22,6 +23,20 @@ sub version {
     ${$self->get_package_symbol('$VERSION')};
 }
 
+sub authority {  
+    my $self = shift;
+    ${$self->get_package_symbol('$AUTHORITY')};
+}
+
+sub identifier {
+    my $self = shift;
+    join '-' => (
+        $self->name,
+        ($self->version   || ()),
+        ($self->authority || ()),
+    );
+}
+
 1;
 
 __END__
@@ -44,6 +59,10 @@ Class::MOP::Module - Module Meta Object
 
 =item B<version>
 
+=item B<authority>
+
+=item B<identifier>
+
 =back
 
 =head1 AUTHORS
